﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;


public class GroupVisual : MonoBehaviour {

	// Studient prefabs
	[SerializeField]
	private List<GameObject> m_studientPrefabs;

	// Radius used for studiens group
	[SerializeField]
	private float m_radius = 2.0f;

	// Current number of studient
	private int m_currentStudientCount = 0;

	Vector3 m_lastPosition;
	bool m_isFirstUpdate = true;

	// Use this for initialization
	void Start () {
	}
	
	// Update is called once per frame
	void Update () {

		if (!m_isFirstUpdate)
		{
			Vector3 dir = new Vector3(transform.position.x - m_lastPosition.x, transform.position.y - m_lastPosition.y, 0.0f);
			dir.Normalize();
			if (dir.magnitude != 0.0f)
			{
				SetChildrenDirection(dir);
			}

		}

		m_lastPosition.Set (transform.position.x, transform.position.y, transform.position.z);
		m_isFirstUpdate = false;

	}

	// Set the studient count
	public void SetStudientCount(int studientCount)
	{

		if (studientCount > m_currentStudientCount)
		{
			// Add the new studients
			for (int i = m_currentStudientCount; i < studientCount; i++)
			{
				//int prefabIndex = (int)(Random.Range(0, 1.0f) * m_studientPrefabs.Count);
				int prefabIndex = (int)(i % m_studientPrefabs.Count);

				GameObject newStudient = Instantiate(m_studientPrefabs[prefabIndex], transform.position, transform.rotation) as GameObject;
				newStudient.transform.parent = transform;
			}

			m_currentStudientCount = studientCount;
			ReorderChildren();
		}
		else if (studientCount < m_currentStudientCount)
		{
			// Destroy the children
			int childToRemove = m_currentStudientCount - studientCount;
			for(int i = 0; i < childToRemove; i++) 
			{
				Destroy(this.transform.GetChild(i).gameObject);
			}

			m_currentStudientCount = studientCount;
			ReorderChildren();
		}
	}

	// Reorder the studients around the group
	private void ReorderChildren()
	{

		if (m_currentStudientCount == 0)
			return;

		float angleIncrement = (2 * Mathf.PI) / m_currentStudientCount;

		for (int i = 0; i < this.transform.childCount; i++)
		{
			Vector3 newPosition = new Vector3();
			newPosition.Set(m_radius * Mathf.Cos(angleIncrement * i), m_radius * Mathf.Sin(angleIncrement * i), 0.0f);
			// Set the position of the child relative to its parent
			this.transform.GetChild(i).localPosition = newPosition;

		}
	}


	// Set the children angle
	private void SetChildrenDirection(Vector3 dir)
	{
		//Quaternion rotation = Quaternion.LookRotation(dir, new Vector3(0.0f, 0.0f, -1.0f));
		//Debug.Log("dir " + dir.x + " " + dir.y + " " + dir.z);
		float parentAngle = 0;
		Vector3 outAxis = Vector3.zero;
		transform.rotation.ToAngleAxis(out parentAngle, out outAxis);

		float angle = Mathf.Atan2(dir.y, dir.x) * Mathf.Rad2Deg;
		if (outAxis.z < 0)
			angle += 180;

		/*
		GameObject test = GameObject.Find("test");
		if (test != null)
		{
			float l = 10;
			Vector3 newPos = new Vector3(transform.position.x + l * Mathf.Cos (angle * Mathf.Deg2Rad), 
			                             transform.position.y + l * Mathf.Sin (angle * Mathf.Deg2Rad),
			                             0.0f);
			test.transform.position = newPos;
		}
		//*/

		angle -= parentAngle;
		Quaternion rotation = Quaternion.AngleAxis(angle, Vector3.forward);

		for (int i = 0; i < this.transform.childCount; i++)
		{
			this.transform.GetChild(i).localRotation = rotation;
		}
	}

}
