﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class PathFollower : MonoBehaviour {

	[SerializeField]
	private float m_moveSpeed = 0.0f;

	[SerializeField]
	private List<Transform> m_path = null;
	[SerializeField]
	private int m_currentPathIndex = -1;

	[SerializeField]
	private Vector2 m_moveDirection = Vector2.zero;

	public delegate void OnDestinationReached();
	private OnDestinationReached m_onDestinationReached = null;

	private CameraManager m_cameraManager;

	[SerializeField]
	private float m_cameraSizeDuringSelection = 27.0f;


	// Use this for initialization
	void Start () {
		EndMove ();
	}

	public void StartMove(List<Transform> path, OnDestinationReached onDestinationReached, CameraManager cameraManger)
	{
		System.Diagnostics.Debug.Assert(path != null);
		System.Diagnostics.Debug.Assert(path.Count > 0);
		System.Diagnostics.Debug.Assert(m_path == null);
		System.Diagnostics.Debug.Assert(m_currentPathIndex == -1);
		
		m_path = path;

		m_currentPathIndex = 0;
		m_moveDirection = path[0].position - gameObject.transform.position;
		m_moveDirection.Normalize();

		m_onDestinationReached = onDestinationReached;

		m_cameraManager = cameraManger;

		enabled = true;
	}

	void EndMove()
	{
		//TODO: clear path list
		enabled = false;
	}
	
	// Update is called once per frame
	void Update ()
	{
		float deltaTime = Time.deltaTime;

		System.Diagnostics.Debug.Assert(m_path != null);
		System.Diagnostics.Debug.Assert(m_moveSpeed >= 0);

		Vector3 move = (deltaTime * m_moveSpeed) * m_moveDirection;
		
		Transform current = gameObject.transform;
		current.Translate(move, Space.World);

		if (m_cameraManager != null)
		{
			m_cameraManager.OnCameraTargetPosition(gameObject.transform.position, m_cameraSizeDuringSelection);
		}

		Transform destination = m_path[m_currentPathIndex];
		Vector2 toDestination = destination.position - current.position;
		float dotMoveDir = Vector2.Dot(m_moveDirection, toDestination);
		if (dotMoveDir <= 0.0f)
		{
			int lastPathIndex = m_path.Count - 1;
			if (m_currentPathIndex != lastPathIndex)
			{
				++m_currentPathIndex;
				Transform nextDestination = m_path[m_currentPathIndex];
				m_moveDirection = nextDestination.position - current.position;
				m_moveDirection.Normalize();
			}
			else
			{
				if (m_onDestinationReached != null)
				{
					m_onDestinationReached();
				}
				
				EndMove();
			}
		}
	}
}