﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class RoomSelector : MonoBehaviour {

	public delegate void OnRoomSelectionDone(List<Room> roomSequence);
	private OnRoomSelectionDone m_onRoomSelectionDone = null;

	LevelManager m_levelManager = null;

	[SerializeField]
	List<Room> m_selectedRooms = null;

	[SerializeField]
	private bool m_cameraFollowSelection = false;

	[SerializeField]
	private float m_cameraSizeDuringSelection = 73.0f;


	// Use this for initialization
	void Start () {
		enabled = false;
	}
	
	// Update is called once per frame
	void Update () {
		m_onRoomSelectionDone(m_selectedRooms);
		enabled = false;
	}

	public Room StartSelection(LevelManager levelManager, Room[] rooms, OnRoomSelectionDone onRoomSelectionDone)
	{
		System.Diagnostics.Debug.Assert(levelManager != null);
		System.Diagnostics.Debug.Assert(onRoomSelectionDone != null);
		System.Diagnostics.Debug.Assert(m_levelManager == null);
		System.Diagnostics.Debug.Assert(m_onRoomSelectionDone == null);

		m_levelManager = levelManager;
		m_onRoomSelectionDone = onRoomSelectionDone;

		Room startRoom = PickStartRoom(rooms);

		m_selectedRooms = new List<Room>();
		m_selectedRooms.Add(startRoom);

		// Notify the camera manager of the start position
		if (levelManager.GetCameraManager() != null && m_cameraFollowSelection)
		{
			levelManager.GetCameraManager().OnCameraTargetPosition(startRoom.transform.position, m_cameraSizeDuringSelection);
		}

		return startRoom;
	}

	void EndSelection()
	{
		m_levelManager = null;
		enabled = true;
	}

	Room PickStartRoom(Room[] rooms)
	{
		RoomStartable[] startableRooms = this.GetComponentsInChildren<RoomStartable>();
		System.Diagnostics.Debug.Assert(startableRooms.Length > 0);
		
		int startRoomIndex = Random.Range(0, startableRooms.Length - 1);
		Room startRoom = rooms[startRoomIndex].GetComponent<Room>();
		System.Diagnostics.Debug.Assert(startRoom != null);
		System.Diagnostics.Debug.Assert(startRoom.GetComponent<RoomEndable>() == null);
		
		return startRoom;
	}
	
	public bool SelectNextRoom(Room clickedRoom)
	{
		System.Diagnostics.Debug.Assert(m_levelManager != null);
		System.Diagnostics.Debug.Assert(m_selectedRooms.Count > 0);
		
		bool selected = false;

		if (m_levelManager != null)
		{
			Room lastSelectedRoom = m_selectedRooms[m_selectedRooms.Count-1];
			
			if (clickedRoom != lastSelectedRoom)
			{
				RoomConnection roomConnection = m_levelManager.GetConnectionBetweenRooms(lastSelectedRoom, clickedRoom);
				if (roomConnection != null)
				{
					m_selectedRooms.Add(clickedRoom);
					
					Debug.Log("Room " + clickedRoom.name + " selected");
					clickedRoom.OnSelectionSuccess();

					// Notify the camera manager
					if (m_levelManager.GetCameraManager() != null && m_cameraFollowSelection)
					{
						m_levelManager.GetCameraManager().OnCameraTargetPosition(clickedRoom.transform.position, m_cameraSizeDuringSelection);
					}
					
					selected = true;

					bool isExit = (clickedRoom.GetComponent<RoomEndable>() != null);
					if (isExit)
					{
						EndSelection();
					}
				}
				else
				{
					Debug.Log("Room " + clickedRoom.name + " can t be selected: Can t find connection with Room " + lastSelectedRoom.name);
					clickedRoom.OnSelectionFailed();
				}
			}
			else
			{
				Debug.Log("Room " + clickedRoom.name + " is alread selected");
				clickedRoom.OnSelectionFailed();
			}
		}
		else
		{
			Debug.Log("Room " + clickedRoom.name + " can t be selected:" + "Exit already selected");

		}
		
		return selected;
	}
}
